@@ -12,7 +12,7 @@
 ** This file contains C code routines that are called by the parser
 ** to handle INSERT statements in SQLite.
 **
-** $Id: insert.c,v 1.26 2004/09/21 14:34:43 matt Exp $
+** $Id: insert.c,v 1.27 2004/10/12 09:01:16 matt Exp $
 */
 #include "sqliteInt.h"
 
@@ -616,8 +616,6 @@ void sqlite3Insert(
     }
   }
 
-  sqlite3EndWriteOperation(pParse);
-
   /*
   ** Return the number of rows inserted.
   */
@@ -1007,6 +1005,7 @@ void sqlite3OpenTableAndIndices(
   assert( v!=0 );
   sqlite3VdbeAddOp(v, OP_Integer, pTab->iDb, 0);
   sqlite3VdbeAddOp(v, op, base, pTab->tnum);
+  VdbeComment((v, "# %s", pTab->zName));
   sqlite3VdbeAddOp(v, OP_SetNumColumns, base, pTab->nCol);
   for(i=1, pIdx=pTab->pIndex; pIdx; pIdx=pIdx->pNext, i++){
     sqlite3VdbeAddOp(v, OP_Integer, pIdx->iDb, 0);
