@@ -12,7 +12,7 @@
 ** This file contains C code routines that are called by the parser
 ** to handle UPDATE statements.
 **
-** $Id: update.c,v 1.26 2004/09/21 14:34:46 matt Exp $
+** $Id: update.c,v 1.27 2004/10/12 09:01:20 matt Exp $
 */
 #include "sqliteInt.h"
 
@@ -253,9 +253,7 @@ void sqlite3Update(
     */
     sqlite3VdbeAddOp(v, OP_Dup, 0, 0);
     if( !isView ){
-      sqlite3VdbeAddOp(v, OP_Integer, pTab->iDb, 0);
-      sqlite3VdbeAddOp(v, OP_OpenRead, iCur, pTab->tnum);
-      sqlite3VdbeAddOp(v, OP_SetNumColumns, iCur, pTab->nCol);
+      sqlite3OpenTableForReading(v, iCur, pTab);
     }
     sqlite3VdbeAddOp(v, OP_MoveGe, iCur, 0);
 
@@ -432,8 +430,6 @@ void sqlite3Update(
     sqlite3VdbeAddOp(v, OP_Close, oldIdx, 0);
   }
 
-  sqlite3EndWriteOperation(pParse);
-
   /*
   ** Return the number of rows that were changed.
   */
